/*
	graphics.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <SDL.h>
#include "SDL_rotozoom.h"
#include "SDL_gfxPrimitives.h"
#include "calc.h"
#include "graphics.h"
#include "ship.h"

SDL_Surface* screen; //Main drawing surface
Uint16 black,red,green,blue,white,grey; //Colour keys, defined at initialisation
extern pers me;
sprite sprites[SPRITECOUNT]; //Main sprite storage bank
long stx[32],sty[32]; //Co-ordinates of stars
const long rmod=RAND_MAX/200; //Modifier to get the rand output into suitable star co-ordinates

void initgraphics()
{
	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER);
	screen=SDL_SetVideoMode(600,400,8,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_HWPALETTE);
	sdlnulltest(screen);
	SDL_WM_SetCaption("Star Voyager","Star Voyager");
	black=SDL_MapRGB((*screen).format,0,0,0);
	red=SDL_MapRGB((*screen).format,255,0,0);
	green=SDL_MapRGB((*screen).format,0,255,0);
	blue=SDL_MapRGB((*screen).format,0,0,255);
	grey=SDL_MapRGB((*screen).format,32,32,32);
	white=SDL_MapRGB((*screen).format,200,200,200);
	for(int i=0;i<SPRITECOUNT;i++)
	{
		sprites[i].self=i;
		sprites[i].imem=false;
	}
	SDL_ShowCursor(0);
}

void drawinterface()
{
	short astx,asty; //Temporary star co-ordinates on screen
	SDL_Rect view={0,0,400,400}; //Viewer rect
	SDL_Rect panel={400,0,200,400}; //Instrument panel rect
	SDL_Rect stpx; //Rect for drawing the star pixel
	
	SDL_SetClipRect(screen,NULL);
	SDL_FillRect(screen,&view,black);
	SDL_FillRect(screen,&panel,blue);
	stpx.w=1;
	stpx.h=1;
	for(short i=0;i<32;i++)
	{
		astx=stx[i]-me.x+200;
		asty=sty[i]-me.y+200;
		if(astx<1|astx>400|asty<1|asty>400)
		{
			astx=2*rand()/rmod;
			asty=2*rand()/rmod;
			stx[i]=astx+me.x-200;
			sty[i]=asty+me.y-200;
		}
		stpx.x=astx;
		stpx.y=asty;
		pixelColor(screen,astx,asty,0xFFFFFFFF);
	}
	SDL_SetClipRect(screen,&view);
}

void update()
{
	SDL_Flip(screen);
}

void sdlnulltest(SDL_Surface* test)
{
	if(test==NULL)
	{
		fprintf(stderr,"Program error: %s\n",SDL_GetError());
		exit(1);
	}
}

void sprite::load(bool rgen)
{
	SDL_Surface* tmp; //Temporary holding place while sprites are converted
	char path[33]; //Path to load sprite from
	short lim; //Limit, as defined by rgen
	
	sprintf(path,"data/sprites/%i.bmp",self);
	rots[0]=SDL_LoadBMP(path);
	if(rgen)
		lim=36;
	else
		lim=1;
	for(short i=0,j=360;i<lim;i++)
	{
		if(i>0)
			rots[i]=rotozoomSurface(rots[0],j,1.0,0);
		sdlnulltest(rots[i]);
		SDL_SetColorKey(rots[i],SDL_SRCCOLORKEY,black);
		hw[i]=(*rots[i]).w/2;
		hh[i]=(*rots[i]).h/2;
		j-=10;
		tmp=SDL_ConvertSurface(rots[i],screen->format,SDL_HWSURFACE);
		SDL_FreeSurface(rots[i]);
		rots[i]=tmp;
	}
	imem=true;
}

void sprite::draw(short x,short y,short rot)
{
	SDL_Rect dst; //Destination rect
	
	if(!imem)
		load(true);
	dst.x=x-hw[rot];
	dst.y=y-hh[rot];
	SDL_BlitSurface(rots[rot],NULL,screen,&dst);
}


