/*
	ship.cpp
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

#include <SDL.h>
#include <stdio.h>
#include <string.h>
#include "calc.h"
#include "frag.h"
#include "equip.h"
#include "graphics.h"
#include "ship.h"

pers me; //User
extern equip equips[];
extern sprite sprites[];
ship ships[SHIPCOUNT]; //Main ship list

void testship()
{
	me.indx=0;
	newship(0,100000,100000,1);
	newship(0,100010,100100,1);
	ships[1].enem=0;
	ships[0].enem=1;	
}

void initships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		ships[i].all=-1;
	}
}

void drawships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1)
		{
			ships[i].draw();
		}
	}
}

void doships()
{
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all!=-1)
		{
			ships[i].physics();
			if(i!=me.indx)
				ships[i].autonav();
			ships[i].maintain();
		}	
	}
	ships[me.indx].control();
	me.x=ships[me.indx].x;
	me.y=ships[me.indx].y;
}

short newship(short indx,long x,long y,short all)
{
	char path[33]; //Path to load ship template from
	
	for(short i=0;i<SHIPCOUNT;i++)
	{
		if(ships[i].all==-1)
		{
			sprintf(path,"data/ships/%hd.shp",indx);
			ships[i].load(path);
			ships[i].x=x;
			ships[i].y=y;
			ships[i].all=all;
			ships[i].hrx=0;
			ships[i].hry=0;
			ships[i].vel.ang=0;
			ships[i].vel.rad=0;
			ships[i].hrsp=0;
			ships[i].shd=0;
			ships[i].pow=ships[i].mpow;
			ships[i].frnd=-1;
			ships[i].enem=-1;
			ships[i].plnt=-1;
			return i;
		}
	}
	return -1;
}

void ship::draw()
{
	short ax,ay; //Position on screen
	short rot; //Rotation frame to use

	ax=x-me.x+200;
	ay=y-me.y+200;
	if(!(ax>450||ax<-50||ay>450||ay<-50))
	{
		rot=vel.ang/10;
		sprites[spr].draw(ax,ay,rot);
	}
}

void ship::physics()
{
	vct mov; //Resultant motion
	
	mov=poltovct(vel,true);
	hrx+=mov.xx;
	hry+=mov.yy;
	vect.xx=hrx/50;
	vect.yy=hry/50;
	x+=vect.xx;
	y+=vect.yy;
	if(hrx<-50)
		hrx=50+hrx%50;
	else
		hrx=hrx%50;
	if(hry<-50)
		hry=50+hry%50;
	else
		hry=hry%50;
}

void ship::control()
{
	Uint8* keys; //Keyboard state
	
	SDL_PumpEvents();
	keys=SDL_GetKeyState(NULL);
	if(keys[SDLK_LEFT])
		turn(-1);
	if(keys[SDLK_RIGHT])
		turn(+1);
	if(keys[SDLK_UP])
		accel(+1);
	if(keys[SDLK_DOWN])
		accel(-1);
	if(keys[SDLK_SPACE])
		shoot(1);
}

void ship::turn(short dir)
{
	if(dir==-1)
	{
		vel.ang-=trn;
		if(vel.ang<0)
			vel.ang=(vel.ang+360)%360;
	}
	if(dir==1)
	{
		vel.ang+=trn;
		if(vel.ang>359)
			vel.ang=vel.ang%360;
	}
}

void ship::accel(short dir)
{
	if(dir==-1)
		hrsp-=aip;
	if(dir==+1)
		hrsp+=aip;
	vel.rad+=hrsp/50;
	if(hrsp<-50)
		hrsp=50+hrsp%50;
	else
		hrsp=hrsp%50;
	if(vel.rad>mip)
		vel.rad=mip;
	if(vel.rad<0)
		vel.rad=0;
}

void ship::autonav()
{
	long tx,ty; //Target co-ordinates
	pol dpol; //Desired polar to target
	vct vect; //Vector to target
	short dd; //Directional difference
	short tol; //Tolerance
	short trg; //Target
	bool hst; //Hostile approach?
	
	if(frnd!=-1)
	{
		hst=false;
		trg=frnd;
	
	}
	else
	{
		if(enem!=-1)
		{
			hst=true;
			trg=enem;
		}
		else
		{
			return;
		}
	}	
	tx=ships[trg].x+ships[trg].vect.xx;
	ty=ships[trg].y+ships[trg].vect.yy;
	vect.xx=tx-x;
	vect.yy=ty-y;
	dpol=vcttopol(vect);
	tol=trn*10;
	dd=(dpol.ang-vel.ang);
	if(dd>180)
		dd=dd-360;
	if(dd<-180)
		dd=dd+360;
	if(dd<tol&&dd>-tol)
	{
		dd=0;
		if(!hst)
			dpol.rad-=50;
		dpol.rad*=10;
		dpol.rad/=aip;
		dpol.rad+=ships[trg].vel.rad;
	}
	else
	{
		dpol.rad=0;
	}
	if(dd>0)
		turn(+1);
	if(dd<0)
		turn(-1);
	if(vel.rad<dpol.rad)
		accel(+1);
	if(vel.rad>dpol.rad)
		accel(-1);
}

void ship::save(char* path)
{
	FILE* strm; //Filestream to use
	
	strm=fopen(path,"w+");
	fprintf(strm,"%s\n%hd\n%hd\n%hd\n%hd\n%ld\n%ld\n",cls,spr,sspr,fspr,all,x,y);
	fprintf(strm,"%hd\n%ld\n%hd\n%ld\n%ld\n%ld\n%ld\n",vel.ang,vel.rad,trn,mip,aip,mwp,awp);
	fprintf(strm,"%ld\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n",srng,mss,pow,rpow,mpow,shd,rshd,mshd);
	fprintf(strm,"%hd\n%hd\n%hd\n",frnd,enem,plnt);
	for(short i=0;i<32;i++)
		fprintf(strm,"\n%hd\n%ld\n%hd\n%hd\n%hd\n",eqp[i].pos.ang,eqp[i].pos.rad,eqp[i].indx,eqp[i].rdy,eqp[i].cap);
	fclose(strm);
}

void ship::load(char* path)
{
	FILE* strm; //Filestream to use
	
	strm=fopen(path,"r");
	fscanf(strm,"%33s\n%hd\n%hd\n%hd\n%hd\n%ld\n%ld\n",cls,&spr,&sspr,&fspr,&all,&x,&y);
	fscanf(strm,"%hd\n%ld\n%hd\n%ld\n%ld\n%ld\n%ld\n",&vel.ang,&vel.rad,&trn,&mip,&aip,&mwp,&awp);
	fscanf(strm,"%ld\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n%hd\n",&srng,&mss,&pow,&rpow,&mpow,&shd,&rshd,&mshd);
	fscanf(strm,"%hd\n%hd\n%hd\n",&frnd,&enem,&plnt);
	for(short i=0;i<32;i++)
		fscanf(strm,"\n%hd\n%ld\n%hd\n%hd\n%hd\n",&eqp[i].pos.ang,&eqp[i].pos.rad,&eqp[i].indx,&eqp[i].rdy,&eqp[i].cap);
	fclose(strm);
}

void ship::shoot(short wep)
{
	equip foc; //Equipment item in focus
	frag out; //Frag scratchpad
	vct vtmp; //Vector scratchpad
	pol ptmp; //Polar scratchpad
	pol trg; //Target
	short acov; //Angle coverage
	short dif; //Angle difference
	short rng; //Range
	bool cov; //Covered by angle and range?		
	
	vtmp.xx=ships[enem].x-x;
	vtmp.yy=ships[enem].y-y;
	trg=vcttopol(vtmp);	
	for(short i=0;i<32;i++)
	{
		foc=equips[eqp[i].indx];
		cov=false;
		if(wep==1)
		{
			acov=45;
			rng=foc.rng*foc.cap;
		}
		if(wep==2)
		{
			acov=10;
			rng=foc.rng;
		}
		if(trg.rad<=rng)
		{
			dif=trg.ang-(vel.ang+eqp[i].pos.ang);
			if(dif>180)
				dif=dif-360;
			if(dif<-180)
				dif=dif+360;
			if(dif<=acov&&dif>=-acov)
				cov=true;		
		}
		if(cov&&eqp[i].rdy==0&&foc.typ==wep&&pow>=foc.pow)
		{
			out.typ=wep;
			out.spr=foc.spr;
			ptmp=eqp[i].pos;
			ptmp.ang=(ptmp.ang+vel.ang)%360;
			vtmp=poltovct(ptmp,false);
			out.x=x+vtmp.xx;
			out.y=y+vtmp.yy;
			ptmp.ang=trg.ang;
			if(wep==1)
				ptmp.rad=foc.cap;
			out.vel=poltovct(ptmp,false);
			out.vel.xx+=vect.xx;
			out.vel.yy+=vect.yy;
			out.rot=trg.ang/10;		
			out.rng=foc.rng;
			pow-=foc.pow;
			newfrag(&out);
			eqp[i].rdy=foc.rdy;
		}
	}
}

void ship::maintain()
{
	for(short i=0;i<32;i++)
	{
		if(eqp[i].rdy>0)
			eqp[i].rdy--;
	}
	if(pow<mpow)
		pow+=rpow;
}
