/*
	ship.h
	
	(c) Richard Thrippleton, July 2001
	Licensing terms are in the 'LICENSE' file
	If that file is not included with this source then permission is not given to use this source in any way whatsoever.
*/

struct pers //Personal data for human user, only one instantiation needed
{
	short indx; //Owned ship
	long x,y; //Point of view

};

struct emp //Equipment emplacement on the ship
{
	pol pos; //Position (ang=-1 indicates not external)
	short indx; //Equipment item index
	short rdy; //Readiness
	short cap; //Capacity
};

class ship //Definition of the spaceship type in game
{
	public:
	
	char cls[33]; //Ship class 
	short spr; //Sprite index
	short sspr; //Shield sprite
	short fspr; //Frag sprite
	short all; //Alliance (0 means dead)
	long x,y; //Co-ordinates 
	
	long hrx,hry; //Hi-res X, subdivides the co-ords into -500 - 500. NOT to be used by any function except 'physics', allows smooth movement at low speed
	pol vel; //Velocity
	vct vect; //Caches velocity vector for use by other functions
	long hrsp; //Hi-res speed, subdivided -500-500. Only to be used by the 'accel' function
	short trn; //Turning speed
	long mip; //Maximum impulse velocity (<100)
	long aip; //Impulse acceleration
	long mwp; //Maximum warp velocity
	long awp; //Warp acceleration
	long srng; //Sensor range
	short mss; //Mass
	short pow; //Current power level
	short rpow; //Power regeneration rate
	short mpow; //Maximum power level
	short shd; //Shield level
	short rshd; //Shield regeneration rate
	short mshd; //Maximum shield level
	short frnd; //Friend target
	short enem; //Enemy target
	short plnt; //Planetary target
			
	emp eqp[32]; //Equipment
	
	void draw(); //Draw ship
	void physics(); //Execute physics on the ship, such as motion
	void control(); //Processes keyboard input and applies it to the ship
	void turn(short dir); //Turns the ship, -1 for left, +1 for right
	void accel(short dir); //Accelerates the ship, -1 for back, +1 for forward
	void autonav(); //Executes auto-pilot considerations on ship
	void save(char* path); //Saves the ship data to a file
	void load(char* path); //Load ship data from a file
	void shoot(short wep); //Shoot at target with wep: 1 Phasers  2 Torpedo
	void maintain(); //Does routine work such as recharging shields and power, reloading weapons
};

void testship();
void initships(); //Initialises the ship array, nulling them all
void drawships(); //Goes through all ships, drawing them
void doships(); //Handle events on all ships such as physics, targetting behaviour etc.
short newship(short indx,long x,long y,short all); //Creates a new ship, returning its index

const short SHIPCOUNT=128; //Total number of ships to be implemented in the game
